
#include "OptionWindow.hpp"
#include "SimpleMailer.hpp"

#define MARGIN 8
#define BUTTON_LABEL_OK     "Save"
#define BUTTON_LABEL_CANCEL "Cancel"

//
// Constructor definition.
// The constructor of the parent class is called for various initializations.
//
OptionWindow::OptionWindow(Void) : SFRTitleWindow(
    SFRApplication::GetInstance(), SFXGraphics::GetDeviceRectangle(), "Option")
{
    // Set up data is obtained from the application class
    SFXConfigPtr option = SimpleMailer::GetOption();

    // Key handler registration
    if (static_try()) {
        static_throw(RegisterHandler(SFEVT_KEY, HANDLER_AFTER, HANDLER_FUNCTION(OnKey)));
    }

    // Get the font height
    SInt16 fontHeight = SFXGraphics::GetFontHeight(AEE_FONT_NORMAL);

    SFXRectangle rectLeft(MARGIN, MARGIN, 100, fontHeight + 2);
    SFXRectangle rectRight(GetContentWorld());

    rectRight.Deflate(MARGIN, MARGIN);
    rectRight.AddLeft(rectRight.GetWidth() - 70);
    rectRight.SetHeight(fontHeight + 2);

    rectLeft.SetRight(rectRight.GetLeft() - MARGIN);

    // Create labels and text controls
    ::new SFRLabelControl(this, rectLeft, "SMTP Server");
    ::new SFRLabelControl(this, rectRight, "Port");

    rectLeft.AddY(MARGIN + fontHeight);
    rectRight.AddY(MARGIN + fontHeight);

    _textControlSMTPServer = ::new SFREditboxControl(this,
        rectLeft, option->ReadSFXAnsiString(OPTION_SMTPSERVER, ""));
    //Set text input mode: letters
    _textControlSMTPServer->SetInputMode(AEE_TM_LETTERS);
    _textControlSMTPServer->RegisterHandler(SREVT_CONTROL, HANDLER_BEFORE, HANDLER_FUNCTION(OnEditboxHandler));


    _textControlSMTPPort = ::new SFREditboxControl(this, rectRight, SFXWideString::Format("%d",
                                 option->ReadUInt16(OPTION_SMTPPOPT, 25)));
    //Set text input mode: numbers
    _textControlSMTPPort->SetInputMode(AEE_TM_NUMBERS);
    _textControlSMTPPort->RegisterHandler(SREVT_CONTROL, HANDLER_BEFORE, HANDLER_FUNCTION(OnEditboxHandler));

    rectLeft.AddY(MARGIN + fontHeight + 8);
    rectRight.AddY(MARGIN + fontHeight + 8);

    ::new SFRLabelControl(this, rectLeft, "POP3 Server");
    ::new SFRLabelControl(this, rectRight, "Port");

    rectLeft.AddY(MARGIN + fontHeight);
    rectRight.AddY(MARGIN + fontHeight);

    _textControlPOP3Server = ::new SFREditboxControl(this, rectLeft, option->ReadSFXAnsiString(OPTION_POP3SERVER, ""));
    //Set text input mode: letters
    _textControlPOP3Server->SetInputMode(AEE_TM_LETTERS);
    _textControlPOP3Server->RegisterHandler(SREVT_CONTROL, HANDLER_BEFORE, HANDLER_FUNCTION(OnEditboxHandler));

    _textControlPOP3Port = ::new SFREditboxControl(this, rectRight,
                                 SFXWideString::Format("%d", option->ReadUInt16(OPTION_POP3PORT, 110)));
    //Set text input mode: numbers
    _textControlPOP3Port->SetInputMode(AEE_TM_NUMBERS);
    _textControlPOP3Port->RegisterHandler(SREVT_CONTROL, HANDLER_BEFORE, HANDLER_FUNCTION(OnEditboxHandler));

    rectLeft.AddY(MARGIN + fontHeight + 8);
    rectRight.AddY(MARGIN + fontHeight + 8);

    rectRight.SetLeft(GetContentWorld().GetWidth() / 2);
    rectLeft.SetRight(rectRight.GetLeft() - MARGIN);

    ::new SFRLabelControl(this, rectLeft, "User");
    ::new SFRLabelControl(this, rectRight, "Password");

    rectLeft.AddY(MARGIN + fontHeight);
    rectRight.AddY(MARGIN + fontHeight);

    _textControlUser = ::new SFREditboxControl(this, rectLeft,
        option->ReadSFXAnsiString(OPTION_USER, ""));
    //Set text input mode: letters
    _textControlUser->SetInputMode(AEE_TM_LETTERS);
    _textControlUser->RegisterHandler(SREVT_CONTROL, HANDLER_BEFORE, HANDLER_FUNCTION(OnEditboxHandler));

   _textControlPassword = ::new SFREditboxControl(this, rectRight, option->ReadSFXAnsiString(OPTION_PASSWORD, ""));
    //Password Unproclaimed display
    _textControlPassword->SetProperties(TP_PASSWORD);
    //Set text input mode: letters
    _textControlPassword->SetInputMode(AEE_TM_LETTERS);


    _textControlPassword->RegisterHandler(SREVT_CONTROL, HANDLER_BEFORE, HANDLER_FUNCTION(OnEditboxHandler));

    rectLeft.AddY(MARGIN + fontHeight + 8);
    rectLeft.SetWidth(GetContentWorld().GetWidth() - 2 * MARGIN);

    // Create the checkbox
    _checkboxEraseMail = ::new SFRCheckboxControl(this, rectLeft, "Delete mail");
    if (_checkboxEraseMail != null) {
        _checkboxEraseMail->SetStatusCheck(option->ReadBool(OPTION_ERASEMAIL, false));
    }

    rectLeft.AddY(MARGIN + fontHeight+4);
    rectLeft.SetWidth(SFXGraphics::MeasureText(AEE_FONT_NORMAL, BUTTON_LABEL_OK) + 2 * MARGIN);

    // Create buttons
    SFRButtonControlPtr buttonOk = ::new SFRButtonControl(this, rectLeft, BUTTON_LABEL_OK);

    rectLeft.AddX(rectLeft.GetWidth() + MARGIN);
    rectLeft.SetRight(GetContentWorld().GetRight() - MARGIN);

    SFRButtonControlPtr buttonCancel = ::new SFRButtonControl(this, rectLeft, BUTTON_LABEL_CANCEL);

    if (_textControlSMTPServer != null) {
        // The maximum size of the text control is set
        _textControlSMTPServer->SetMaxSize(64);
        // Focus is applied
        _textControlSMTPServer->SetStatusFocus(true);
    }
    if (_textControlSMTPPort != null) {
        _textControlSMTPPort->SetMaxSize(5);
    }
    if (_textControlPOP3Server != null) {
        _textControlPOP3Server->SetMaxSize(64);
    }
    if (_textControlPOP3Port != null) {
        _textControlPOP3Port->SetMaxSize(5);
    }
    if (_textControlUser != null) {
        _textControlUser->SetMaxSize(64);
    }
    if (_textControlPassword != null) {
        _textControlPassword->SetMaxSize(64);
    }

    // Button handlers registration
    if (static_try()) {
        static_throw(buttonOk->RegisterHandler(SREVT_CONTROL, HANDLER_BEFORE, HANDLER_FUNCTION(OnButtonControlOk)));
    }
    if (static_try()) {
        static_throw(buttonCancel->RegisterHandler(SREVT_CONTROL, HANDLER_BEFORE, HANDLER_FUNCTION(OnButtonControlCancel)));
    }

    // "Select" key press handler registration (for text controls)
    if (static_try()) {
        static_throw(RegisterHandler(SFEVT_KEY, AVK_SELECT, HANDLER_AFTER, HANDLER_FUNCTION(SelectHandler)));
    }

    return;
}

//
// Destructor definition.
// Nothing special is done here.
//
OptionWindow::~OptionWindow(Void)
{
    return;
}

//
// Key handler.
//
HANDLER_IMPLEMENT_BOOLEVENT(OptionWindow, OnKey, event)
{
    SFRResponderPtr target;

    switch (event.GetP16()) {
        case AVK_CLR:
            target = GetFront(TYPE_SFRCONTROL, ATTRIBUTE_SFREDITBOXCONTROL, STATUS_TARGET);
            if (target != null) {
                target->SetStatusTarget(false);
            }
            else{
                // If the clear key is pushed, close the window
                return Invoke(SFXEvent(SREVT_RESPONDER_TERMINATE, SRP16_TERMINATE_INVOKE, true));
            }
        case AVK_UP:
            // Focus is moved to previous
            FocusPrevious();
            return true;
        case AVK_DOWN:
            // Focus is moved to next
            FocusNext();
            return true;
    }
    return false;
}

//
// "Ok" button press handler.
//
HANDLER_IMPLEMENT_VOIDCONTROL(OptionWindow, OnButtonControlOk, result, control)
{
    unused(result);
    unused(control);

    // Save data entered in the text controls of OptionWindow
    SFXConfigPtr option = SimpleMailer::GetOption();
    option->WriteSFXAnsiString(OPTION_SMTPSERVER, _textControlSMTPServer->GetText());
    option->WriteUInt16(OPTION_SMTPPOPT, static_cast<UInt16>(_textControlSMTPPort->GetText().AsUInt32()));
    option->WriteSFXAnsiString(OPTION_POP3SERVER, _textControlPOP3Server->GetText());
    option->WriteUInt16(OPTION_POP3PORT, static_cast<UInt16>(_textControlPOP3Port->GetText().AsUInt32()));
    option->WriteSFXAnsiString(OPTION_USER, _textControlUser->GetText());
    option->WriteSFXAnsiString(OPTION_PASSWORD, _textControlPassword->GetText());
    option->WriteBool(OPTION_ERASEMAIL, _checkboxEraseMail->GetStatusCheck());
    option->Save();

    // Close window
    Invoke(SFXEvent(SREVT_RESPONDER_TERMINATE, SRP16_TERMINATE_INVOKE, true));
}

//
// "Cancel" button press handler.
//
HANDLER_IMPLEMENT_VOIDCONTROL(OptionWindow, OnButtonControlCancel, result, control)
{
    unused(result);
    unused(control);

    // Close window
    Invoke(SFXEvent(SREVT_RESPONDER_TERMINATE, SRP16_TERMINATE_INVOKE, true));
}

HANDLER_IMPLEMENT_VOIDCONTROL(OptionWindow, OnEditboxHandler, result, control)
{
    SFRResponderPtr next;

    control->SetStatusTarget(false);
    if (result) {
        next = control->GetNext(TYPE_WILDCARD, ATTRIBUTE_WILDCARD);
        if (next != null) {
            next->SetStatusFocus(true);
            next->SetStatusTarget(true);
        }
    }
    return;
}
