
#include "ReceivedWindow.hpp"
#include "SimpleMailer.hpp"

//
// Constructor definition.
// The constructor of the parent class is called for various initializations.
//
ReceivedWindow::ReceivedWindow(Void) : SFRPlainWindow(
    SFRApplication::GetInstance(), SFXGraphics::GetDeviceRectangle()), _index(0)
{
    // Drawing handler registration
    if (static_try()) {
        static_throw(RegisterHandler(SREVT_RESPONDER_RENDER,
            SRP16_RENDER_CONTENT, HANDLER_BEFORE,
            HANDLER_FUNCTION(OnRenderContent)));
    }

    // Key handler registration
    if (static_try()) {
        static_throw(RegisterHandler(SFEVT_KEY, HANDLER_AFTER,
            HANDLER_FUNCTION(OnKey)));
    }

    // Virtual area registration (for scrolling on screen)
    SetVirtualBound(SFXGraphics::GetDeviceRectangle().AddHeight(1000));
    return;
}

//
// Destructor definition.
// Nothing special is done here.
//
ReceivedWindow::~ReceivedWindow(Void)
{
    return;
}

//
// Window drawing handler.
//
HANDLER_IMPLEMENT_VOIDRENDER(ReceivedWindow, OnRenderContent, graphics)
{
    SFXPropertyPtr mail = SimpleMailer::GetMail();

    // Paint background white
    graphics->FillRectangle(GetVirtualWorld(), SFXRGBColor(0xFF, 0xFF, 0xFF, 0x00));

    // Display of mail data
    graphics->DrawString(mail->GetValue(_index), GetVirtualWorld(),
                       SFXRGBColor(0x00, 0x00, 0x00, 0x00), IDF_ALIGN_LEFT);
    return;
}

//
// Key handler.
//
HANDLER_IMPLEMENT_BOOLEVENT(ReceivedWindow, OnKey, event)
{
    SFXPropertyPtr mail = SimpleMailer::GetMail();

    switch (event.GetP16()) {
        case AVK_CLR: 
			{
        // Close window
            return Invoke(SFXEvent(SREVT_RESPONDER_TERMINATE, SRP16_TERMINATE_INVOKE, true));
			}
        // Focus is moved
        case AVK_UP:
            // Scroll up
            Scroll(SFXSize(0, -40));
            return true;
        case AVK_DOWN:
            // Scroll down
            Scroll(SFXSize(0, 40));
            return true;
        case AVK_LEFT:
            // Switch displayed mail to previous
            if (_index > 0) {
                --_index;
                InvalidateVirtual();
            }
            return true;
        case AVK_RIGHT:
            // Switch displayed mail to next
            if (_index < mail->GetSize() - 1) {
                ++_index;
                InvalidateVirtual();
            }
            return true;
    }
    return false;
}
