
#include "SendingWindow.hpp"
#include "SimpleMailer.hpp"

#define MARGIN 11
#define BUTTON_LABEL_OK     "Send"
#define BUTTON_LABEL_CANCEL "Cancel"

//
// Constructor definition.
// The constructor of the parent class is called for various initializations.
//
SendingWindow::SendingWindow(Void) : SFRTitleWindow(SFRApplication::GetInstance(), SFXGraphics::GetDeviceRectangle(), "Send mail")
{
    // Key handler registration
    if (static_try()) {
        static_throw(RegisterHandler(SFEVT_KEY, HANDLER_AFTER, HANDLER_FUNCTION(OnKey)));
    }

    SInt16 fontHeight = SFXGraphics::GetFontHeight(AEE_FONT_NORMAL);
    SFXRectangle rectLeft(MARGIN, MARGIN, 0, fontHeight + 2);
    SFXRectangle rectRight(GetContentWorld());

    rectLeft.SetWidth(SFXGraphics::MeasureText(AEE_FONT_NORMAL, "From") + MARGIN);

    rectRight.Deflate(MARGIN, MARGIN);
    rectRight.SetLeft(rectLeft.GetRight());
    rectRight.SetHeight(fontHeight + 2);

    rectLeft.SetRight(rectRight.GetLeft() - MARGIN);

    // Create text controls and labels
    ::new SFRLabelControl(this, rectLeft, "From");

    _textControlFrom = ::new SFREditboxControl(this, rectRight, "");
    // Set character input mode to alphabet
    _textControlFrom->SetInputMode(AEE_TM_LETTERS);
    _textControlFrom->RegisterHandler(SREVT_CONTROL, HANDLER_BEFORE, HANDLER_FUNCTION(OnEditboxHandler));

    rectLeft.AddY(MARGIN + fontHeight);
    rectRight.AddY(MARGIN + fontHeight);

    ::new SFRLabelControl(this, rectLeft, "To");
    _textControlTo = ::new SFREditboxControl(this, rectRight, "");
    // Set character input mode to alphabet
    _textControlFrom->SetInputMode(AEE_TM_LETTERS);
    _textControlTo->RegisterHandler(SREVT_CONTROL, HANDLER_BEFORE, HANDLER_FUNCTION(OnEditboxHandler));

    rectLeft.AddY(MARGIN + fontHeight);
    rectRight.AddY(MARGIN + fontHeight);

    ::new SFRLabelControl(this, rectLeft, "Sub");
    _textControlSubject = ::new SFREditboxControl(this, rectRight, "");
    _textControlSubject->RegisterHandler(SREVT_CONTROL, HANDLER_BEFORE, HANDLER_FUNCTION(OnEditboxHandler));

    rectLeft.AddY(MARGIN + fontHeight);

    ::new SFRLabelControl(this, rectLeft, "Body");

    rectLeft.AddY(MARGIN + fontHeight);

    rectLeft.SetBottom(GetContentWorld().GetBottom() - rectLeft.GetHeight() - 2 * MARGIN);
    rectLeft.SetRight(GetContentWorld().GetRight() - MARGIN);

    _textControlBody = ::new SFREditboxControl(this, rectLeft, "");
    _textControlBody->RegisterHandler(SREVT_CONTROL, HANDLER_BEFORE, HANDLER_FUNCTION(OnEditboxHandler));

    rectRight.SetX(rectLeft.GetX());
    rectRight.SetY(rectLeft.GetBottom() + MARGIN);
    rectRight.SetWidth(SFXGraphics::MeasureText(AEE_FONT_NORMAL, BUTTON_LABEL_OK) + 2 * MARGIN);

    // Create buttons
    SFRButtonControlPtr buttonOk = ::new SFRButtonControl(this, rectRight, BUTTON_LABEL_OK);

    rectRight.AddX(rectRight.GetWidth() + MARGIN);
    rectRight.SetRight(GetContentWorld().GetRight() - MARGIN);

    SFRButtonControlPtr buttonCancel = ::new SFRButtonControl(this, rectRight, BUTTON_LABEL_CANCEL);

    if (_textControlFrom != null) {
        // Set max size of text control
        _textControlFrom->SetMaxSize(255);
        // Focus is applied
        _textControlFrom->SetStatusFocus(true);
    }
    if (_textControlTo != null) {
        _textControlTo->SetMaxSize(255);
    }
    if (_textControlSubject != null) {
        _textControlSubject->SetMaxSize(255);
    }
    if (_textControlBody != null) {
        _textControlBody->SetMaxSize(SINT16_MAXIMUM);
    }

    // Button handlers registration
    if (static_try()) {
        static_throw(
            buttonOk->RegisterHandler(
                SREVT_CONTROL, HANDLER_BEFORE,
                HANDLER_FUNCTION(OnButtonControlOk)));
    }
    if (static_try()) {
        static_throw(
            buttonCancel->RegisterHandler(
                SREVT_CONTROL, HANDLER_BEFORE,
                HANDLER_FUNCTION(OnButtonControlCancel)));
    }

    // "Select" key press handler registration (for text controls)
    if (static_try()) {
        static_throw(
            RegisterHandler(
                SFEVT_KEY, AVK_SELECT, HANDLER_AFTER,
                HANDLER_FUNCTION(SelectHandler)));
    }

    return;
}

//
// Destructor definition.
// Nothing special is done here.
//
SendingWindow::~SendingWindow(Void)
{
    return;
}

//
// Key handler
//
HANDLER_IMPLEMENT_BOOLEVENT(SendingWindow, OnKey, event)
{
    SFRResponderPtr target;

    switch (event.GetP16()) {
        // If the clear key is pushed
        case AVK_CLR:
            target = GetFront(TYPE_SFRCONTROL, ATTRIBUTE_SFREDITBOXCONTROL, STATUS_TARGET);
            if (target != null) {
                target->SetStatusTarget(false);
            }
            else{
                // If the clear key is pushed, close the window
                return Invoke(SFXEvent(SREVT_RESPONDER_TERMINATE, SRP16_TERMINATE_INVOKE, true));
            }
        case AVK_UP:
            // Focus is moved to previous
            FocusPrevious();
            return true;
        case AVK_DOWN:
            // Focus is moved to next
            FocusNext();
            return true;
    }
    return false;
}

//
// "Ok" button press handler.
//
HANDLER_IMPLEMENT_VOIDCONTROL(SendingWindow, OnButtonControlOk, result, control)
{
    unused(result);
    unused(control);

    SFCError error;
    // Setup data is obtained from the application class
    SFXConfigPtr option = SimpleMailer::GetOption();

    // Class that holds an email message
    SFXMailMessage message;

    // Set the "From" address
    message.SetFromField(_textControlFrom->GetText());

    // Set the "To" address
    message.AddToField(_textControlTo->GetText());

    // Set the "Subject"
    message.SetSubjectField(_textControlSubject->GetText());

    // Set the message body
    message.SetBody(_textControlBody->GetText());

    // Set SMTP server address and port number (the domain name is automatically resolved)
    _sender.SetServer(SFXSocketAddress(option->ReadSFXAnsiString(OPTION_SMTPSERVER, ""),
                      option->ReadUInt16(OPTION_SMTPPOPT, 25)));

    // Send mail and register a callback function
    error = _sender.SendMessage(&message, CALLBACK_FUNCTION(SMTPCallback));

    if (error == SFERR_NO_ERROR) {
        // Create info dialog
        _dialog = ::new SFRMessageDialog(SFXGraphics::GetDeviceRectangle().Deflate(20, 100), "Sending...", "");
    }
    else {
        // Create info dialog
        _dialog = ::new SFRMessageDialog(SFXGraphics::GetDeviceRectangle().Deflate(20, 100), "Failed during transmission", "OK");
        if (_dialog != null) {
            // Dialog handler registration
            _dialog->RegisterHandler(SREVT_DIALOG, HANDLER_BEFORE, HANDLER_FUNCTION(OnDialog));
        }
    }
}

//
// "Cancel" button press handler.
//
HANDLER_IMPLEMENT_VOIDCONTROL(SendingWindow, OnButtonControlCancel, result, control)
{
    unused(result);
    unused(control);

    // Close window
    Invoke(SFXEvent(SREVT_RESPONDER_TERMINATE, SRP16_TERMINATE_INVOKE, true));
}

//
// Callback executed when mail sending is completed.
//
CALLBACK_IMPLEMENT_SFXSMTPSENDER(SendingWindow, SMTPCallback, error)
{
    SFXWideString message;

    if (error == SFERR_NO_ERROR) {
        message.Set("Sending succeeded.");
    }
    else {
        message.Set("Sending failed.");
    }
    if (_dialog != null) {
        // Close "Sending" dialog
        _dialog->Invoke(SFXEvent(SREVT_RESPONDER_TERMINATE, SRP16_TERMINATE_INVOKE, true));
    }

    // Create dialog
    _dialog = ::new SFRMessageDialog(SFXGraphics::GetDeviceRectangle().Deflate(20, 100), message, "OK");
    if (_dialog != null) {
        // Registration of dialog handler
        _dialog->RegisterHandler(SREVT_DIALOG, HANDLER_BEFORE, HANDLER_FUNCTION(OnDialog));
    }

    // Redrawing
    SFRApplication::GetInstance()->Invoke(
        SFXEvent(SREVT_RESPONDER_RENDER, SRP16_RENDER_INVOKE, false));
}

//
// Dialog handler.
//
HANDLER_IMPLEMENT_VOIDDIALOG(SendingWindow, OnDialog, result, dialog)
{
    // Close window
    dialog->Invoke(SFXEvent(SREVT_RESPONDER_TERMINATE, SRP16_TERMINATE_INVOKE, true));
    // Close window
    Invoke(SFXEvent(SREVT_RESPONDER_TERMINATE, SRP16_TERMINATE_INVOKE, true));
    // Redraw
    SFRApplication::GetInstance()->Invoke(SFXEvent(SREVT_RESPONDER_RENDER, SRP16_RENDER_INVOKE, false));
}

HANDLER_IMPLEMENT_VOIDCONTROL(SendingWindow, OnEditboxHandler, result, control)
{
    SFRResponderPtr next;

    control->SetStatusTarget(false);
    if (result) {
        next = control->GetNext(TYPE_WILDCARD, ATTRIBUTE_WILDCARD);
        if (next != null) {
            next->SetStatusFocus(true);
            next->SetStatusTarget(true);
        }
    }
    return;
}
